<?php

namespace FreenetRouter\Libraries\ServiceManagers;

/**
 * @author Jakub Petržílka <kubapet@lbcfree.net>
 */
class WebServerManager implements IServiceManager
{  
    private function isApacheActive()
    {
        $apacheResult = exec("/etc/init.d/apache2 status");
        return (strpos($apacheResult,'is running') !== false);   
    }
    
    private function isPhpFpmActive()
    {
        $phpFpmResult = exec("/etc/init.d/php5-fpm status");
        return (strpos($phpFpmResult,'is running') !== false);        
    }
    
    public function isActive()
    {   
        return ($this->isApacheActive()&&$this->isPhpFpmActive());
    }
    
    private function isApacheEnabled()
    {
        return is_link("/etc/rc2.d/S91apache2");
    }
    
    private function isPhpFpmEnabled()
    {
        return is_link("/etc/rc2.d/S91php5-fpm");
    }

    public function isEnabled()
    {
        return ($this->isApacheEnabled()&&$this->isPhpFpmEnabled());
    }

    public function setActive(boolean $active)
    {
        if (($active)&&(!$this->isApacheActive()))
        {
            exec("sudo /etc/init.d/apache2 start");
        }
        else if ((!$active)&&($this->isApacheActive()))
        {
            exec("sudo /etc/init.d/apache2 stop");
        }        
        
        if (($active)&&(!$this->isPhpFpmActive()))
        {
            exec("sudo /etc/init.d/php5-fpm start");
        }
        else if ((!$active)&&($this->isApacheActive()))
        {
            exec("sudo /etc/init.d/php5-fpm stop");
        }                
    }
    
    private function setApacheEnabled(boolean $enabled)
    {
        $files = array("/etc/rc0.d/K09apache2","/etc/rc1.d/K09apache2",
            "/etc/rc2.d/S91apache2","/etc/rc3.d/S91apache2",
            "/etc/rc4.d/S91apache2","/etc/rc5.d/S91apache2",
            "/etc/rc6.d/K09apache2");
        if ($enabled) 
        {
            foreach ($files as $file) 
            {
                exec("sudo /bin/ln -s /etc/init.d/apache2 ".$file);
            }
        } 
        else
        {
            exec("sudo /bin/rm -f ".implode(" ",$files));
        }                
    }
    
    private function setPhpFpmEnabled(boolean $enabled)
    {
        $files = array("/etc/rc0.d/K09php5-fpm","/etc/rc1.d/K09php5-fpm",
            "/etc/rc2.d/S91php5-fpm","/etc/rc3.d/S91php5-fpm",
            "/etc/rc4.d/S91php5-fpm","/etc/rc5.d/S91php5-fpm",
            "/etc/rc6.d/K09php5-fpm");
        if ($enabled) 
        {
            foreach ($files as $file) 
            {
                exec("sudo /bin/ln -s /etc/init.d/php5-fpm ".$file);
            }
        } 
        else
        {
            exec("sudo /bin/rm -f ".implode(" ",$files));
        }         
    }

    public function setEnabled(boolean $enabled)
    {
        $this->setApacheEnabled($enabled);
        $this->setPhpFpmEnabled($enabled);
    }

}
