<?php

namespace Phem\Libraries\ResourceLoading;

use Phem\Core\Collection;
use Phem\Core\Object;
use Phem\Environment\Resource;
use ReflectionClass;

/**
 * @author Jakub Petržílka <petrzilka@czweb.net>
 */
abstract class ThirdPartyResourceLoader extends Object
{

    private $resources;

    public function init()
    {
        $this->resources = new Collection();
        $this->load();
        return $this->resources;
    }
    
    /**
     * Adds a resource file or resource directory to be processed
     * and loaded on the page
     * 
     * In case of directory you can specify if it should be processed recursively
     * 
     * @param string $name Name of the resource
     * @param string $path Relative path to the file or directory
     * @param ResourceType $type Type of resource
     * @param boolean $recursive Recursive processing of dirs (default true)
     * @param int $priority Resource priority (default 0)
     */
    final protected function addResource(
            $name, $path, $type, $recursive = true, $priority = 0
        )
    {
        $classInfo = new ReflectionClass($this);
        $dirname = dirname($classInfo->getFileName());
        $pathPrefix = str_replace(getcwd(),"",$dirname);

        $res = new Resource();
        $res->setName($name);
        $res->setPath($pathPrefix . DS . $path);
        $res->setType($type);
        $res->setRecursiveLoad($recursive);
        $res->setPriority($priority);

        $this->resources->add($res);
    }

    protected abstract function load();
}
