<?php

namespace Phem\Libraries\Security\Authorization;

use Phem\Environment\EnvironmentManager;
use Phem\Libraries\Security\Model\Common\Task;
use Phem\Libraries\Security\Model\Common\User;
use Phem\Libraries\Security\Model\MySQL\Permission;
use Phem\Libraries\Security\Model\MySQL\User as MySQLUser;

class MySQLAuthorizationProvider implements IAuthorizationProvider
{

    public function canTask(User $user, Task $task)
    {       
        
        $em = EnvironmentManager::getEntityManager();

        $mysqlTask = $em->getRepository("Phem\Libraries\Security\Model\MySQL\Task")
                ->findOneBy(array("name" => $task->getName()));
        
        $mysqlCtrl = $em->getRepository("Phem\Libraries\Security\Model\MySQL\Controller")
                ->findOneBy(array("name" => $task->getController()->getName()));        


        $taskId = ($mysqlTask != null) ? $mysqlTask->getId() : null;
        $ctrlId = ($mysqlCtrl != null) ? $mysqlCtrl->getId() : null;

        if ($user instanceof MySQLUser)
        {
            $user = $em->merge($user);
        }
        else
        {
            $user = $em->getRepository("Phem\Libraries\Security\Model\MySQL\User")
                    ->findOneBy(array("username"=>$user->getUsername()));
        }
        
        if ($user == null)
        {
            return IAuthorizationProvider::PERMISSION_UNKNOWN;
        }

        foreach ($user->getGroups() as $group)
        {

            $groupPermissionsQuery = $em->createQuery(
                    "select p from Phem\Libraries\Security\Model\MySQL\Permission p
                     where p.group = :group 
                     and ((p.task = :task) or (p.controller = :controller))");
            $groupPermissionsQuery->setParameters(array(
                'group' => $group->getId(),
                'task' => $taskId,
                'controller' => $ctrlId,
            ));
            $groupPermissionsQuery->setMaxResults(3);
            $groupPermissions = $groupPermissionsQuery->getResult();
            
           /*echo $taskId . 't<br/>';
            echo $ctrlId . 'c<br/>';
            echo $group->getId() . 'g<br/>';*/
            

            if (count($groupPermissions) > 2)
                throw new AuthorizationException("Duplicate ACL entries for group " . $group->getName() . ' and task ' . $mysqlTask->getName());
            
            
            $ctrlEnabled = IAuthorizationProvider::PERMISSION_UNKNOWN;
            foreach ($groupPermissions as $permission)
            {
                if (($permission->getTask() != null) && ($permission->getType() == Permission::ALLOW))
                {
                    return IAuthorizationProvider::PERMISSION_ALLOWED;
                }
                else if (($permission->getTask() != null) && ($permission->getType() == Permission::DENY))
                {
                    return IAuthorizationProvider::PERMISSION_DENIED;
                }
                else if (($permission->getController() != null) && ($permission->getType() == Permission::ALLOW))
                {
                    $ctrlEnabled = IAuthorizationProvider::PERMISSION_ALLOWED;
                }
                else if (($permission->getController() != null) && ($permission->getType() == Permission::DENY))
                {
                    $ctrlEnabled = IAuthorizationProvider::PERMISSION_DENIED;
                }                
            }
            return $ctrlEnabled;

        }
        return false;
    }

}