<?php

namespace Phem\Libraries\Security\Management;

use Phem\Core\Collection;
use Phem\Environment\EnvironmentManager;
use Phem\Libraries\Security\Model\Common\Controller;
use Phem\Libraries\Security\Model\Common\Group;
use Phem\Libraries\Security\Model\Common\Permission;
use Phem\Libraries\Security\Model\Common\Task;
use Phem\Libraries\Security\Model\MySQL\Permission as MySQLPermission;
use Phem\ToolSuite;

/**
 * @author kubapet
 */
class MySQLPermissionManager implements IPermissionManager
{
    public function addPermission(Permission $permission)
    {
        $newPermission = new MySQLPermission();
        
        ToolSuite::populateObjectFromAnother($permission, $newPermission);
        
        $em = EnvironmentManager::getEntityManager();
        $em->persist($newPermission);
        $em->flush();
        $em->detach($newPermission);
    }

    public function getGroups(Task $task)
    {
        $em = EnvironmentManager::getEntityManager();
        $permissions = $em->getRepository("Phem\Libraries\Security\Model\MySQL\Permission")
                ->findBy(array("task" => $task));

        $controller = $task->getController();
        $controllerPermissions = $em->getRepository("Phem\Libraries\Security\Model\MySQL\Permission")
                ->findBy(array("controller" => $controller));

        $groups = new Collection();

        foreach ($controllerPermissions as $permission)
        {
            if ($permission->getType == Permission::ALLOW)
            {
                $permissionGroup = $permission->getGroup();
                $groups->put($permissionGroup->getName(), $permissionGroup);
            }
        }

        foreach ($permissions as $permission)
        {
            $permissionGroup = $permission->getGroup();
            if ($permission->getType == Permission::ALLOW)
            {
                $groups->put($permissionGroup->getName(), $permissionGroup);
            }
            else
            {
                $groups->removeKey($permissionGroup->getName());
            }
        }
        return $groups;
    }

    public function getTasks(Group $group)
    {
        $em = EnvironmentManager::getEntityManager();
        $permissions = $em->getRepository("Phem\Libraries\Security\Model\MySQL\Permission")
                ->findBy(array("group" => $group));

        $tasks = new Collection();
        
        foreach ($permissions as $permission)
        {
            if ($permission->getController() != null)
            {
                if ($permission->getType == Permission::ALLOW)
                {
                    $permissionController = $permission->getController();
                    $tasks->join($permissionController->getAvailableTasks());
                }
            }
        }        
        
        foreach ($permissions as $permission)
        {
            if ($permission->getTask() != null)
            {
                if ($permission->getType == Permission::ALLOW)
                {
                    $permissionTask = $permission->getTask();
                    $tasks->put($permissionTask->getName(), $permissionTask);
                }
                else
                {
                    $tasks->remove($permissionTask->getName());
                }
            }
        }
        
        return $tasks;
    }

    public function removePermission(Group $group, Task $task)
    {
        $em = EnvironmentManager::getEntityManager();
        $permission = $em->getRepository("Phem\Libraries\Security\Model\MySQL\Permission")
                ->findOneBy(array("group" => $group, "task" => $task));

        $em->remove($permission);
        $em->flush();
    }

    public function removeControllerPermission(Group $group, Controller $controller)
    {
        $em = EnvironmentManager::getEntityManager();
        $permission = $em->getRepository("Phem\Libraries\Security\Model\MySQL\Permission")
                ->findOneBy(array("group" => $group, "controller" => $controller));

        $em->remove($permission);
        $em->flush();
    }

    public function getPermissions(Permission $prototype = null, $orderBy = null, $limit = null)
    {
        $em = EnvironmentManager::getEntityManager();
        if ($prototype != null)
        {
            $criteria = (array) $prototype;
            return $em->getRepository("Phem\Libraries\Security\Model\MySQL\Permission")
                            ->findBy($criteria, $orderBy, $limit);
        }
        else
        {
            $q = $em->getRepository("Phem\Libraries\Security\Model\MySQL\Permission")
                    ->createQueryBuilder('permissionQuery');
            $q->setMaxResults($limit);
            return $q->getQuery()->getResult();
        }        
    }

}
