<?php

namespace Phem\Libraries\Security\Management;

use Phem\Environment\EnvironmentManager;
use Phem\Libraries\Security\Model\Common\Controller;
use Phem\Libraries\Security\Model\Common\Task;
use Phem\Libraries\Security\Model\MySQL\Controller as MySQLController;
use Phem\Libraries\Security\Model\MySQL\Task as MySQLTask;
use Phem\ToolSuite;

/**
 * @author kubapet
 */
class MySQLTaskManager extends TaskManager
{

    public function getRegisteredTasksOfController(Controller $controller)
    {
        $em = EnvironmentManager::getEntityManager();
        $em->getRepository("Phem\Libraries\Security\Model\MySQL\Task")
                ->findBy(array('controller' => $controller));
    }

    public function getRegistredControllers()
    {
        $em = EnvironmentManager::getEntityManager();
        $q = $em->getRepository("Phem\Libraries\Security\Model\MySQL\Controller")
                ->createQueryBuilder('controllerQuery');
        return $q->getQuery()->getResult();
    }

    public function getRegistredTasks()
    {
        $em = EnvironmentManager::getEntityManager();
        $q = $em->getRepository("Phem\Libraries\Security\Model\MySQL\Task")
                ->createQueryBuilder('tasksQuery');
        return $q->getQuery()->getResult();
    }

    public function registerController(Controller $controller)
    {
        $newController = new MySQLController();
        ToolSuite::populateObjectFromAnother($controller, $newController);
        $em = EnvironmentManager::getEntityManager();
        $em->persist($newController);
        $em->flush();
        $em->detach($newController);
    }

    public function registerTask(Task $task)
    {
        $newTask = new MySQLTask();
        ToolSuite::populateObjectFromAnother($task, $newTask);
        $em = EnvironmentManager::getEntityManager();
        $em->persist($newTask);
        $em->flush();
        $em->detach($newTask);
    }

    public function unregisterController(Controller $controller)
    {
        $em = EnvironmentManager::getEntityManager();
        $em->remove($controller);
        $em->flush();
    }

    public function unregisterTask(Task $task)
    {
        $em = EnvironmentManager::getEntityManager();
        $em->remove($task);
        $em->flush();
    }

}
