<?php

namespace Phem\Libraries\Security\Management;

use Phem\Environment\EnvironmentManager;
use Phem\Libraries\Security\Model\Common\Group;
use Phem\Libraries\Security\Model\Common\User;
use Phem\Libraries\Security\Model\MySQL\Group as MySQLGroup;
use Phem\Libraries\Security\Model\MySQL\User as MySQLUser;
use Phem\ToolSuite;

/**
 * @author kubapet
 */
class MySQLUserManager implements IUserManager
{

    public function addGroup(Group $group)
    {
        $newGroup = new MySQLGroup();
        ToolSuite::populateObjectFromAnother($group, $newGroup);
        $em = EnvironmentManager::getEntityManager();
        $em->persist($newGroup);
        $em->flush();
        $em->detach($newGroup);
    }

    public function addUser(User $user)
    {
        $newUser = new MySQLUser();
        ToolSuite::populateObjectFromAnother($user, $newUser);
        $em = EnvironmentManager::getEntityManager();
        $em->persist($newUser);
        $em->flush();
        $em->detach($newUser);
    }

    public function getUser($username)
    {
        $em = EnvironmentManager::getEntityManager();
        return $em->getRepository("\Phem\Libraries\Security\Model\MySQL\User")
                        ->findOneBy(array("username" => $username));
    }

    public function getGroup($name)
    {
        $em = EnvironmentManager::getEntityManager();
        return $em->getRepository("\Phem\Libraries\Security\Model\MySQL\Group")
                        ->findOneBy(array("name" => $name));
    }

    public function addUserToGroup(Group $group, User $user)
    {
        $mysqlGroup = $this->getGroup($group->getName());
        $mysqlUser = $this->getUser($user->getUsername());

        if (($mysqlGroup != null) && ($mysqlUser != null))
        {
            /* $membership = new Membership();
              $membership->setUser($mysqlUser);
              $membership->setGroup($mysqlGroup); */
            $mysqlGroup->getUsers()->add($mysqlUser);
            $em = EnvironmentManager::getEntityManager();
            $em->flush();
        }
        else
        {
            throw new UserManagerException("Cannot add user to group, the given user or group is null");
        }
    }

    public function getGroupsOf(User $user)
    {
        $mysqlUser = $this->getUser($user->getUsername());
        if ($mysqlUser != null)
        {
            return $mysqlUser->getGroups();
        }
    }

    public function getUsersOf(Group $group)
    {
        $mysqlGroup = $this->getGroup($group->getName());
        if ($mysqlGroup != null)
        {
            return $mysqlGroup->getUsers();
        }
        else
        {
            throw new UserManagerException("User doesn't exist");
        }
    }

    public function isMemberOf(User $user, Group $group)
    {
        $mysqlGroup = $this->getGroup($group->getName());
        $mysqlUser = $this->getUser($user->getUsername());
        
        //why this was here?
        //$mysqlGroup->getUsers()->add($mysqlUser);
        $em = EnvironmentManager::getEntityManager();
        $membership = $em->getRepository("Phem\Libraries\Security\Model\MySQL\Membership")
                ->findOneBy(array('user' => $mysqlUser, 'group' => $mysqlGroup));

        return $membership != null ? true : false;
    }

    public function modifyGroup(Group $group)
    {
        if ($group instanceof MySQLGroup)
        {
            $em = EnvironmentManager::getEntityManager();
            $em->merge($group);
            $em->flush();
            $em->detach($group);
        }
        else
        {
            throw new UserManagerException('Group doesnt exist yet');
        }
    }

    public function modifyUser(User $user)
    {
        if ($user instanceof MySQLUser)
        {
            $em = EnvironmentManager::getEntityManager();
            $em->merge($user);
            $em->flush();
            $em->detach($user);
        }
        else
        {
            throw new UserManagerException('User doesnt exist yet');
        }
    }

    public function removeGroup(Group $group)
    {
        $em = EnvironmentManager::getEntityManager();
        $em->remove($group);
        $em->flush();
    }

    public function removeUser(User $user)
    {
        $em = EnvironmentManager::getEntityManager();
        $em->remove($user);
        $em->flush();
    }

    public function removeUserFromGroup(Group $group, User $user)
    {
        $mysqlGroup = $this->getGroup($group->getName());
        $mysqlUser = $this->getUser($user->getUsername());
        $mysqlGroup->getUsers()->add($mysqlUser);
        $em = EnvironmentManager::getEntityManager();
        $membership = $em->getRepository("Phem\Libraries\Security\Model\MySQL\Membership")
                ->findOneBy(array('user' => $mysqlUser, 'group' => $mysqlGroup));

        if ($membership != null)
        {
            $em->remove($membership);
            $em->flush();
        }
        {
            throw new UserManagerException('Given user is not member of given group');
        }
    }

    public function getGroups(Group $prototype = null, $orderBy = null, $limit = null)
    {
        $em = EnvironmentManager::getEntityManager();
        if ($prototype != null)
        {
            $criteria = (array) $prototype;
            return $em->getRepository("Phem\Libraries\Security\Model\MySQL\Group")
                            ->findBy($criteria, $orderBy, $limit);
        }
        else
        {
            $q = $em->getRepository("Phem\Libraries\Security\Model\MySQL\Group")
                    ->createQueryBuilder('groupsQuery');
            $q->setMaxResults($limit);
            return $q->getQuery()->getResult();
        }
    }

    public function getUsers(User $prototype = null, $orderBy = null, $limit = null)
    {
        $em = EnvironmentManager::getEntityManager();
        if ($prototype != null)
        {
            $criteria = (array) $prototype;
            return $em->getRepository("Phem\Libraries\Security\Model\MySQL\User")
                            ->findBy($criteria, $orderBy, $limit);
        }
        else
        {
            $q = $em->getRepository("Phem\Libraries\Security\Model\MySQL\User")
                    ->createQueryBuilder('usersQuery');
            $q->setMaxResults($limit);
            return $q->getQuery()->getResult();
        }
    }

}
