<?php

namespace Phem\Libraries\Security\Management;

use Phem\Core\Collection;
use Phem\Libraries\Security\Model\Common\Controller;
use Phem\Libraries\Security\Model\Common\Task;
use Phem\ToolSuite;
use ReflectionClass;
use ReflectionMethod;

abstract class TaskManager
{

    public abstract function registerController(Controller $controller);

    public abstract function unregisterController(Controller $controller);

    public abstract function registerTask(Task $task);

    public abstract function unregisterTask(Task $task);

    public abstract function getRegisteredTasksOfController(Controller $controller);

    public abstract function getRegistredTasks();

    public abstract function getRegistredControllers();

    public function getAvailableTasksOfController(Controller $controller)
    {
        $tasks = new Collection();
        $r = new ReflectionClass(APP_NAMESPACE . '\\' . CONTROLLER_DIR . '\\' . $controller->getName() . CONTROLLER_SUFFIX);
        $methods = $r->getMethods(ReflectionMethod::IS_PUBLIC);

        foreach ($methods as $method)
        {
            if ((strpos($method->getDeclaringClass()->getName(),'Phem\\') === false)
                    &&($method->getName()!="__construct"))
            {
                /* @var $method ReflectionMethod */
                $task = new Task();
                $task->setName($method->getName());
                $task->setController($controller);
                $tasks->put($task->getName(), $task);
            }
        }

        return $tasks;
    }

    public function getAvailableTasks()
    {
        $controllers = $this->getAvailableControllers();
        $tasks = new Collection();
        foreach ($controllers as $controller)
        {
            $controllerTasks = $this->getAvailableTasksOfController($controller);
            $tasks->join($controllerTasks);
        }
        return $tasks;
    }

    public function getAvailableControllers()
    {
        $dirContent = scandir(APP_DIR . DS . CONTROLLER_DIR);
        $controllers = new Collection();

        foreach ($dirContent as $dirEntry)
        {
            if ((is_file($dirEntry)) && (ToolSuite::endsWith($dirEntry, CONTROLLER_SUFFIX . '.php')))
            {
                $controllerName = str_replace(CONTROLLER_SUFFIX . '.php', '', $dirEntry);
                $ctrl = new Controller();
                $ctrl->setName($controllerName);
                $controllers->put($ctrl->getName(), $ctrl);
            }
        }
        return $controllers;
    }

}