<?php

namespace Phem\Libraries\Security\Model\Common;

use Phem\Core\Collection;
use Phem\Core\Object;
use Phem\Environment\EnvironmentManager;
use Phem\Libraries\Forms\Annotations\GeneratedValue;
use Phem\ToolSuite;

/**
 * @\Phem\Libraries\Forms\Annotations\Form(defaultGroup="permission")
 * @\Phem\Libraries\Forms\Annotations\Group(name="permission", caption="Oprávnění", order=1)
 */
class Permission extends Object
{
    /* Types */
    const DENY = false;
    const ALLOW = true;

    /** 
     * @GeneratedValue
     * @\Phem\Libraries\Forms\Annotations\Input(readOnly="true")
     */
    protected $id;
    /** 
     * @\Phem\Libraries\Forms\Annotations\Select(caption="Skupina", options="getGroups") 
     */
    protected $group;
    /** 
     * @\Phem\Libraries\Forms\Annotations\Select(caption="Kontroler", options="getRegisteredControllers")
     */
    protected $controller;
    
    /**
     * @\Phem\Libraries\Forms\Annotations\Select(caption="Úloha", options="getRegisteredTasks")
     */
    protected $task;
    
    /** @\Phem\Libraries\Forms\Annotations\Select(caption="Typ", options="getTypes") */
    protected $type;

    public function getId()
    {
        return $this->id;
    }

    public function setId($id)
    {
        $this->id = $id;
    }

    public function getGroup()
    {
        return $this->group;
    }

    public function setGroup($group)
    {
        $this->group = $group;
    }

    public function getController()
    {
        return $this->controller;
    }

    public function setController($controller)
    {
        $this->controller = $controller;
    }

    public function getTask()
    {
        return $this->task;
    }

    public function setTask($task)
    {
        $this->task = $task;
    }

    public function getType()
    {
        return $this->type;
    }

    public function setType($type)
    {
        $this->type = $type;
    }
    
    public function save()
    {
        EnvironmentManager::getPermissionManager()->addPermission($this);
    }
    
    public static function getGroups(){
        $groups = EnvironmentManager::getUserManager()->getGroups();
        $groupNames = ToolSuite::getSubCollection($groups, "Name","Id");
        return $groupNames;
    }
    
    public static function getRegisteredControllers(){
        $controllers = EnvironmentManager::getTaskManager()->getRegistredControllers();
        $controllersNames = ToolSuite::getSubCollection($controllers, "Name","Id");
        return $controllersNames;
    }
    
    public static function getRegisteredTasks(){
        $tasks = EnvironmentManager::getTaskManager()->getRegistredTasks();
        $tasksNames = ToolSuite::getSubCollection($tasks, "Name","Id");
        return $tasksNames;
    }
    
    public static function getTypes(){
        $col = new Collection();
        $col->put((string)Permission::ALLOW, "ALLOW");
        $col->put((string)Permission::DENY, "DENY");
        return $col;
    }

}