<?php

/*
 *  Application scope storage backend to be used (php support needed)
 * 
 *  Default: APC
 *  
 * Supported values:
 * 
 *  APC 
 *  XCache
 *  Disabled
 */
define("APPLICATION_SCOPE_STORAGE","Disabled");

/*
 * Application notifications (MessageBus) method
 * 
 * WebSocket server needs to run using standalone socket.run.sh script 
 * which uses cgi-fcgi bridge to inject it to the same php-fpm or hhvm thread pool
 * as the web server is running (this is need to be able to access same SHM 
 * and thus application scope variables).
 * 
 * Comet approach uses traditional pending ajax requests (requires pcntl php supprt,
 * works only on Unix/Linux based platforms). 
 * Requires application scope storage enabled
 * 
 * Default: Both
 * 
 * Supported values:
 * 
 * Both
 * WebSocket 
 * Comet
 * Disabled
 *  
 */
define("MESSAGEBUS_MODE","Disabled");

/*
 *  Puller mode for application notifications
 *  (How to notify MessageBus runing on socket server from ordinary http request)
 *  This has no effect if MESSAGE_BUS is not set to "WebSocket" or "Both"
 * 
 *  ZMQ needs php support and currently doesn't work with HHVM.
 * 
 *  Default: ReactSocket
 * 
 *  Supported values:
 * 
 *  ReactSocket 
 *  ZMQ
 *  Disabled
 */
define("WEBSOCKET_PULLER_MODE","Disabled");

define("ALLOW_ONLINE_CLIENT_NOTIFICATIONS",false);

define("WEBSOCKET_PULLER_BIND_ADDR","127.0.0.1");
define("WEBSOCKET_PULLER_PORT",4000);

define("WEBSOCKET_BIND_ADDR","127.0.0.1");
define("WEBSOCKET_PORT",1337);

/**
 * Defines address and port to which should the client side connect
 * Example: wss://example.com:443 or ws://example.net:80
 */
define("WEBSOCKET_CONNECT_URI","wss://example.com:443");

define("SITE_SUBDIR","");

define("CUSTOM_MAINFRAME",true);

/* Mailer */
define("MAILFROM_ADDRESS","root@example.com");

/* Enable/Disable link translating defined by LinkBuilder */
if (!defined("REWRITE_LINKS"))
{
    define("REWRITE_LINKS", false);
}

/* Root directory of application */
if (!defined("APP_DIR"))
{
    define("APP_DIR",ROOT_DIR.DS.'Application');
}

define("DOCTRINE_DEVEL_MODE",true);
define("TWIG_DEVEL_MODE",true);

define("APP_NAMESPACE",'FreenetRouter');
define("APP_NAME",'FreenetRouter');

/* Controllers directory */
define("CONTROLLER_DIR","Controllers");

define("RESOURCES_SYMLINK", APP_DIR.DS."Resources");

/* Resource directories */
define("CSS_DIR_NAME", 'css');
define("JS_DIR_NAME", 'javascript');
define("IMG_DIR_NAME", 'images');


define("CSS_DIR", RESOURCES_SYMLINK.DS.CSS_DIR_NAME);
define("JS_DIR", RESOURCES_SYMLINK.DS.JS_DIR_NAME);
define("IMG_DIR", RESOURCES_SYMLINK.DS.IMG_DIR_NAME);
define("TEMPLATE_DIR", APP_DIR.DS.'Templates');

/** debug and logs */
//define("DEBUG",true);

define("LOG_LEVEL", serialize(array('log','error', 'warning', 'accept', 'info')));

/* APPLICATION CUSTOM CONFIG */

/** defaults */
define("DEFAULT_CONTROLLER", "Home");
define("DEFAULT_TASK", "showPage");

/** security */
define("LOGIN_REQUIRED", false);
define("LOGIN_POLICY", "DENY");

/** db connection */
define("DB_HOST", "localhost");   
define("DB_DEFAULT_CONFIG",'default');

define("DB_CONFIG",serialize(array(
    'default' => array(
        'name' => 'mydatabase',
        'user' => 'myuser', 
        'password' => 'mypasswd'
    )
)));

define("FPM_INJECT_PORT",9000);
define("FPM_INJECT_HTTP_HOST","localhost");

define("FPDF_FONTPATH", RUN_DIR.DS.'Font'.DS);

mb_internal_encoding("UTF-8");
